function [nd,nf,A,B,reord,CF]=REDKW(A,B,CF,nx,lpd,optred,track,tol);

t1=clock;
tolz=.00000001;
told=10^-16;
itmax=100;

%
%  LREDKW.m: system reduction program for large linear dynamic systems,
%    using a modification of the algorithm implemented in REDKW.m
%
%  The input system is assumed to be written in the form 
%
%  A * d(t+1/t) =  B * d(t) +  C(F) * x(t/t)
%
%  with the vector of variables d(t) being partitioned as
%  
%           | lam(t) |
%  d(t) =   |        |
%           |  k(t)  |
% 
%  where k(t) is predetermined.
%
%  The output is a linear dynamic system of the form
%  
%  0 =  I * f(t) + H * d(t) +  J(F) * x(t/t)
%
%  I * d(t+1/t) = W * d(t) +  PSI(F) * x(t/t)
% 
%  where I is an identity matrix and the output system has
%  a smaller dimension d(t).
%   
%  For compatibility with previouse programs, the system is 
%  output in the A,B,CF form with A=0 (scalar), 
%
%  B =[I H
%      0 W]
% 
%  CF=[J(F)
%      PSI(F)
%
%  All matrices are assumed sparse on input and on output.

% Initial system dimension

% number of flows
nf=0;
% size of unreduced system
ny=size(A,2);
% size of reduced system
nd=ny;
% number of predetermined variables
npd=length(lpd);
nk=npd;
% number of nonpredetermined variables
nlam=ny-npd;
% number of leads (0,1,...)
nlead=size(CF,2)/nx;  

% Tracking matrix for number of zeros isolated each iteration
NZ=[];
% Initialize system matrices
G=[];
H=[];
JF=[];

%%

% BLOCK 1: REORDERING SYSTEM TO PLACE PREDETERMINED VARIABLES LAST

% The code is designed to run even if there are no 
% predetermined variables (as in Cagan money demand
% model or asset pricing models).

ord=1:1:ny;

if npd> 0   % construct a vector of original ordering 
   
   % location matrices for predetermined and nonpredetermined variables
   
   npdloc=ones(1,ny);  % initialize location of nonpredetermined variables
   npdloc(lpd)=0;
   pdloc=ones(1,ny)-npdloc;
   
   % construct a reordering vector
   
    reord=[ord(logical(npdloc)) ord(logical(pdloc))];
 
% Reorder columns of A and B to reflect reordering of variables.

A=A(:,reord);
B=B(:,reord);

else % If there are are no predetermined variables   
   reord=ord;
end

% iteration number
it=0;
% number of zero rows isolated
nz=0;
%%

% indicator for first iteration
first=1;
% indicator for continued reduction
redgo=1;

while redgo==1  & it<itmax  
    
    it=it+1;
    
%%

% BLOCK 2: optionally elimate existing rows of zeros on first iteration
% if first ==1;
% IF THIS IS A FIRST REDUCTION, CHECK FOR ROWS OF ZEROS IN A

if first==1

A=A';
[sa1 sa2]=sort(max(abs(A)));
A=A';
% sa1 contains the absolute value of the biggest element of each row A.
% sa2 contains indices that sort these from lowest to highest.
% determine number of rows of A with effectively zero elements

nz=sum(sa1<tolz);

A=A(sa2,:);   
B=B(sa2,:);   
CF=CF(sa2,:); 

first=0;

end
%%

% BLOCK 3: IF THERE ARE NO ZERO ROWS THEN TRY TO INDUCE SOME USING THE "QR"
% (option 1) OR "SVD" (option 2). 

% optred=1 means QR, optred=2 means SVD

if (nz==0) & optred==1
    
% reorder columns so that predetermined variables occur first
e=[nlam+1:nd 1:nlam];
Q=full(A(:,e));
[Q,A]=qr(Q);

% reorder columns so that nonpredetermined variables occur first
e=[nk+1:nd 1:nk];
A=A(:,e);

% transform dynamic system
Q=Q';
CF=Q*CF;
B=Q*B;

% reorder rows so that zeros occur first
A=A(e,:);
B=B(e,:);
CF=CF(e,:);
clear Q;

% system is now in block lower triangular form. Induce zero rows
% into the system if possible.

[q,a,e]=qr(full(A(1:nlam,1:nlam)),0);

% find biggest element in each row
rr=max(abs(a'));
tol=10^-8;

% number of nonzero rows is "npos"

npos = sum(rr > tol);

if (track>1)
disp(' ')
disp(['On iteration ',num2str(it),' the QR finds'])
disp(['nlam = ',num2str(nlam),' npos = ',num2str(npos)])
disp(['so that ',num2str(nlam-npos),' rows of zeros are induced'])
disp(' ')
end

if npos<nlam

q=q';
B(1:nlam,:)=q*B(1:nlam,:);
CF(1:nlam,:)=q*CF(1:nlam,:);
clear q;

% flip zero rows to top
flip=[npos+1:nlam 1:npos];
A(1:nlam,1:nlam)=a(flip,:);
B(1:nlam,:)=B(flip,:);
CF(1:nlam,:)=CF(flip,:);

% Since the QR is of the form a=q*r*e' but e is written as a permutation
% vector, we need to have a vector that is equivalent to e' so that we
% can calculate the new "a" =r*e'. The following two steps do this.

clear ep;
ep(e)=1:nlam;
A(1:nlam,1:nlam)=A(1:nlam,ep);

nz=nlam-npos;

else   

% no zero rows are found
nz=0;
redgo=0;

end  % if npos<nlam

elseif (nz==0) & optred~=1

	it=it+1;
	
[U A V]=svd(A);

s=diag(A);
tol = nd * s(1) * eps;
npos = sum(s > tol);

nz=nd-npos;

if (track>1)
disp(' ')
disp(['On iteration ',num2str(it),' the QR finds'])
disp(['so that ',num2str(nd-npos),' rows of zeros are induced'])
disp(' ')
end

U=U';
A=A*V';
B=U*B;
CF=U*CF;

clear U;
clear V;

% flips zero rows to top
flip=[npos+1:nd 1:npos];
A=A(flip,:);
B=B(flip,:);
CF=CF(flip,:);

if (nz==0)
    redgo=0;
end

end  % if (nz==0)

%%

if (nz>0)

    NZ=[NZ nz];
    
% BLOCK 4: SYSTEM REQUIRES FURTHER REDUCTION AND ZERO ROWS ARE PRESENT
% AS THE FIRST "nz" ROWS OF A, AS INDUCED BY QR OR SVD 

% Step 2: Determine the number of new flows that can be extracted
% and accordingly manipulate equations of system

% [Q,R,E] = QR(A,0) produces an "economy size" decomposition in which E
%    is a permutation vector, so that A(:,E) = Q*R.  This contrasts 
%    with the standard decomposition A*E=Q*R, where E is a matrix. 


nz=full(nz);

r=full(B(1:nz,1:nlam));
[q,r,e]=qr(r,0);

% Reorder the variables in the model to placed those that are new flows
% at the top of the vector.

e=[e nlam+1:nd];

reordl=reord(nf+1:ny);
reord=[reord(1:nf) reordl(e)];

A=A(:,e);
B=B(:,e);

if size(H,2)>0
 H=H(:,e);
end

% Transform the local block equations so that there is a nonsingular
% matrix attached to the new flows, which involves multiplying by "q-prime"

B(1:nz,:)=q'*B(1:nz,:);
CF(1:nz,:)=q'*CF(1:nz,:);
clear q;

% Determine number of new flows that can be extracted (nonzero rows of r)

rr=abs(diag(r(:,1:size(r,1))));
tol = max(size(r,1)) * rr(1) * eps;  
nnf = sum(rr > tol);

% add lines testing problems

% Determine equations for new flows.

B(1:nnf,:)=r(1:nnf,1:nnf)\B(1:nnf,:);

CF(1:nnf,:)=r(1:nnf,1:nnf)\CF(1:nnf,:);

%  Create new system in form 
%  
%  0 =  G * f(t) + H * d(t) +  J(F) * x(t/t)
%
%  A * d(t+1/t) = B * d(t) +  C(F) * x(t/t)
% 
%  with a smaller dimension d(t) and a larger f(t).

% Expand size of "flow vector"

if (max(size(H)))>0
    
G=[G             H(:,1:nnf)
   zeros(nnf,nf) eye(nnf)];

H=[H(:,nnf+1:nd)
   B(1:nnf,nnf+1:nd)];

old=size(JF,2);
new=size(CF,2);

JF=[JF zeros(nf,new-old)
    CF(1:nnf,:)];

else
    
G=eye(nnf);
H=B(1:nnf,nnf+1:nd);
JF=[CF(1:nnf,:)];

end

% Reduce size of "d" vector
% (calculate smaller A,B,CF matrices)

CF=[CF(nnf+1:nd,:)-B(nnf+1:nd,1:nnf)*CF(1:nnf,:) zeros(nd-nnf,nx)]+...
    [zeros(nd-nnf,nx)  A(nnf+1:nd,1:nnf)*CF(1:nnf,:)];

A=A(nnf+1:nd,nnf+1:nd)-A(nnf+1:nd,1:nnf)*B(1:nnf,nnf+1:nd);

B=B(nnf+1:nd,nnf+1:nd)-B(nnf+1:nd,1:nnf)*B(1:nnf,nnf+1:nd);

% change system dimension parameters

nd=max(size(A));
nlam=nd-nk;
nf=nf+nnf;
nz=0;

end % if nz>0

end

%%

% System is reduced so that A is nonsingular

if (track>1)
disp('reduced system matrix')
disp('rank and size of A')
disp([rank(A) max(size(A))])
end

%  Write system in desired output for
%  
%  0 =  I * f(t) + H * d(t) +  J(F) * x(t/t)
%
%  I * d(t+1/t) = W * d(t) +  PSI(F) * x(t/t)
% 
%  where I is an identity matrix and the output system has
%  a smaller dimension d(t).

W=A\B;
CF=A\CF;

if (size(G,1)>0)
H=G\H;
JF=G\JF;
end

% Output format

A=0;

B=sparse([eye(nf,nf) H
          zeros(nd,nf) W]);

CF=[JF zeros(size(JF,1),size(CF,2)-size(JF,2))
    CF];
