function [M,PI,G,ylist,xlist,lpd,elist]=...
    RESOLKW(system,driver,track,comp,et);

% Rational expectations solution program implementing 
% the various steps described in R.G.King and M.W.Watson
% "System Reduction and Model Solution Algorithms for
% Linear Rational Expectations Models." The inputs and 
% outputs are summarized by the function statement:
%
% [M,PI,G,ylist,xlist,lpd]=resolkw(system,driver,track,comp)
%
% There are two required inputs to the program:
%
%   (1) The file 'system.m' must describe the 
%       linear rational expectations model to be solved.
%       (the input file need not be called 'system.m', but
%        can take any name that the user supplies in the call
%        to the resolkw.m program).  The RE model is:
%
%      A Ey(t+1)|I(t) = B y(t) + C0 x(t) + ... Cn Ex(t+n)|I(t)
%
%       system.m must contain 
%         (a) the matrices A,B,CF=[C0 ... Cn]
%         (b) a vector of predetermined variable locations (lpd)
%         (c) the number of exogenous variables, 
%              which implies the number of leads 
%              using (nlead=cols(CF)/nx)
%         (d) lists of endogenous (ylist) and
%              exogenous (xlist) variables
%
%   (2) The file 'driver.m' must contain the matrices 
%       RHO, Q and Gbar that describe the driving processes
% 
%         x(t) = Q delta(t)
%         delta(t) = RHO * delta(t-1) + Gbar e(t)
% 
%   (3) The program output is a solution in state space form:
%       
%           y(t) = PI * s(t)
%          
%           s(t) = M s(t-1) + G e(t)
%      
%       with s(t) containing the predetermined endogenous variables
%       followed by the delta(t).
%
%   (4) The user may optionally decide on various levels of tracking
%       (there are also some forced error messages). In the inputs, 
%       
%       track=0     no messages
%       track=1     eigenvalue information
%       track=10    extensive information
%
%   (5) The user may optionally decide not to use the 
%       Schur decomposition method.  This involves supply
%       a two element vector that involves [schur nobalance]. 
%       The parameter schur = 0 involves computation via
%       the eigenvector-eigenvalue method; 
%       the parameter nobalance = 1 specifies the
%       computation of the eigenvalues by MATLAB's no balance
%       option, which leads to more accurate computations 
%       in some situations.

if (nargin<3)
   % light tracking is default
   track=1;   
end

% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
% Select details of computational method: 
%    standard eigenvalue (schur=0, nb=0 or not included)
%    unbalanced eigenvalue (schur=0, nb=1)
%    schur (schur=1)
% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

if (nargin<4)
   % Schur decomposition is default
   schur=1;  
   nb=0;
   % QR reductions is default
   optred=2;
elseif (min(size(comp))==1)
   schur=comp(1);
   nb=comp(2);
   optred=1;
else
   schur=comp(1,1);
   nb=comp(1,2);
   optred=comp(2,1);
end

if (track==10)
   % Heavy tracking produces a diary file
   dr=[system,'.out'];
   disp('Tracking option on')
   disp(['diary is ',dr])
   eval(['diary ',dr]);   
end

% *************************************************************
% Step 0: Settings for computational parameters and methods
% *************************************************************

% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
% Choose critical value for roots: 
%      mu is unstable if bcrit*abs(mu) >= 1
% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

bcrit=1/1.00001;
    
    
% *************************************************************
% Step 1: Specifying dynamic system                                           
% *************************************************************

% Run the file 'system.m', which contains the setup of
% the dynamic system

eval(system)

% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
% Optional information on setup of model                        
% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

if (track==10)
   disp('A matrix')
   disp(A)
   disp('B matrix')
   disp(B)
   disp('transformed C(F) matrix')
   disp(CF)
   nlead=COLS(CF)/nx;
   disp(['Number of leads: ',num2str(nlead)])
   disp('Predetermined endogenous variables: ')
   disp(ylist(lpd,:))
   disp('Full vector of endogenous variables')
   disp(ylist) 
   disp('Exogenous variables')
   disp(xlist)
   disp('  ')
   dr=['s',system,'.mat A B CF nx lpd ylist xlist'];
   eval(['save ',dr]); 
   disp(['Original system information saved in ',dr])
   disp(' ')
   disp('running system reduction program (redkw.m)')  
end

% *************************************************************
% Step 2: Call to reduction program                                                
% *************************************************************

% Optional elimination of some model equations

if (nargin>4)
  [s1,s2]=sort(etype);
  disp('solving sparse RE model with some specified non-dynamic equations')
  ss1=num2str(sum(s1==1));
  disp(['associated varables assumed located in first ',ss1,' positions'])
  [A,B,CF,nx,lpd]=PREDKW(A,B,CF,nx,lpd,track,etype);

[ny,junk]=size(A);

end
% Standard KW reduction

[nd,nf,A,B,reord,CF]=REDKW(A,B,CF,nx,optred,2,track);


save CHECK nd nf A B reord CF


% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
% Optional information on output of reduction program                  
% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

if (track==10)
   disp('transformed A matrix')
   disp(A)
   disp('transformed B matrix')
   disp(B)
   disp('Reordering of variables')
   disp(reord)
   disp('transformed C(F) matrix')
   disp(CF)
   disp('flow variables')
   disp(ylist(reord(1:nf),:))
   disp('dynamic variables')
   disp(ylist(reord(nf+1:nf+nd),:))
   disp(' ')
   disp('reordering vector')
    disp('  ')
   dr=['r',system,'.mat nd nf AN BN reord CFN']
   eval(['save ',dr]); 
   disp(['Reduced system information saved in ',dr])
   disp(' ')
   disp('running reduced system transformation program (dynkw.m)') 
end

% **************************************************************
% Step 3: Translation matrices for fundamental dynamic equation
% **************************************************************

[LE,MU]=DYNKW(B,nf,ny,lpd,bcrit,track,schur,nb);

if (track==10)
   disp('reduced system translation completed by dynkw.m')
   disp(' ')
   disp(['The discount factor for eigenvalues is ',num2str(bcrit)])
   disp('unstable eigenvalues are: ')
   disp(diag(MU))
   disp('LE matrix that isolates unstable canonical variables is:')
   disp(LE)
   dr=['d',system,'.mat LE MU']
   eval(['save ',dr]); 
   disp(['Transformed system information saved in ',dr])
   disp(' ')
   disp('running Markov decision rule program (mdrkw.m)') 
end

% *************************************************************
% Step 4: Solution for Markov decision rule                                        
% *************************************************************

% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
%   4.1. Specification of forcing process                 
% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

eval(driver)

if (track==10)

disp('Driving process components (RHO,Q,G) in system specified as:')
disp('        x(t)    = Q delta(t)')
disp('        delta(t) = RHO * delta(t-1) + G e(t)')
disp(RHO)
disp(Q)
disp(Gbar)
end

% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
%   4.2. Call Markov decision rule solution
%          program and get output                                
% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

[M,PI]=MDRKW(B,CF,Q,RHO,LE,MU,reord,nd,lpd,nx,bcrit,schur);

if nargin>4
	% Augment system with results for variables eliminated in PREREDKW.m

load PREDKW.mat

npd=length(lpd);

nlead=size(Kx);
nlead=nlead(2)/nx;
[ny,ns]=size(PI);
ny=ny-nx;
[nz,junk]=size(K);

j=0;

FC=Kx(:,j*nx+1:(j+1)*nx)*Q;

for j=1:nlead-1;
  FC=FC+Kx(:,j*nx+1:(j+1)*nx)*Q*RHO;
end
FC=-FC;


PI=[-K*PI(1:ny,1:npd)  -K*PI(1:ny,npd+1:ns)+FC
    PI];

end

% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
% Augment G matrix for larger system 
% (including endogenous variables)
% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

%number of endogenous states
nlpd=max(ROWS(lpd),COLS(lpd)); %
% number of innovations
ninv=COLS(Gbar);

% Augment matrix mapping shocks to states
G=[zeros(nlpd,ninv)
   Gbar];

% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
%   Optional output information
%   on Markov decision rules                 
% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

if (track==10)
  disp('Markov decision rules computed')
  disp(' ')
  disp(ylist(1:ny,:))
  disp('  ')
  disp('PI matrix: y(t) elements')
  disp('  ')
  disp(PI(1:ny,:))
  disp(xlist(1:nx,:))
  disp('  ')
  disp('PI matrix: x(t) elements')
  disp('  ')
  disp(PI(ny+1:ny+nx,:))
  disp('elements of k(t)')
  disp(ylist(lpd,:))
  disp('  ')
  disp('M matrix:  elements ordered k(t), then delta(t)')
  disp('  ')
  disp(M)
  dr=['m',system,'.mat PI M G']
  eval(['save ',dr]); 
  disp(['Decision rule information saved in ',dr])
  disp(' ')
  disp('model solution completed by resolkw.m')
end

diary off

M=full(M);
PI=full(PI);
G=full(G);




